@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2020.  All Rights Reserved.
+# © Copyright IBM Corp. 2020, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -10,6 +10,7 @@
 
 import logging
 import os
+import uuid
 from pathlib import Path
 
 import dill as pickle
@@ -413,13 +414,18 @@ class FileModelStore:
                 raise Exception(
                     'Serialization of model %s that is supposed to be stored in ModelStore failed.' % model_name) from ex
 
+        model_name = model_name.replace("/",":")
+
         filename = self.path + self.STORE_TABLENAME + model_name
+
         f = open(filename, "wb")
         f.write(model)
         f.close()
 
     def retrieve_model(self, model_name, deserialize=True):
 
+        model_name = model_name.replace("/",":")
+
         filename = self.path + self.STORE_TABLENAME + model_name
 
         model = None
