@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -9,6 +9,7 @@
 # *****************************************************************************
 
 import logging
+import unittest
 import numpy as np
 import pandas as pd
 from sklearn.metrics import r2_score
@@ -22,7 +23,7 @@ from iotfunctions.bif import AggregateWithExpression
 
 from iotfunctions.anomaly import (SaliencybasedGeneralizedAnomalyScore, SpectralAnomalyScore, NoDataAnomalyScore,
                                  FFTbasedGeneralizedAnomalyScore, KMeansAnomalyScore, MatrixProfileAnomalyScore)
-from nose.tools import (assert_true, nottest)
+#from nose.tools import (assert_true, nottest)
 
 logger = logging.getLogger('Test Regressor')
 
@@ -38,7 +39,7 @@ mat = 'TemperatureMatrixProfileScore'
 nod = 'TemperatureNoDataScore'
 
 
-@nottest
+#@nottest
 class DatabaseDummy:
     tenant_id = '###_IBM_###'
     db_type = 'db2'
@@ -124,8 +125,8 @@ def test_anomaly_scores():
     print(comp2)
 
     # assert_true(comp2[spectral] > 0.9)
-    assert_true(comp2[fft] > 0.9)
-    assert_true(comp2[sal] > 0.9)
+    assert (comp2[fft] > 0.9)
+    assert (comp2[sal] > 0.9)
 
     # assert_true(comp2[kmeans] > 0.9)
     df_agg = df_i.copy()
@@ -157,9 +158,9 @@ def test_anomaly_scores():
     #EngineLogging.configure_console_logging(logging.DEBUG)
 
     #aggobj = Aggregation(None, ids=['entity'], timestamp='timestamp', granularity=('5T', (Temperature,), True, 0),
-    #                simple_aggregators=[([Temperature], func_clos, TempDiff)])
+    #                simple_aggregators=[([Temperature], func_clos, [TempDiff])])
     aggobj = Aggregation(None, ids=['entity'], timestamp='timestamp', granularity=('5T', None, True, 0),
-                    simple_aggregators=[([Temperature], func_clos, TempDiff)])
+                    simple_aggregators=[(func_clos, [Temperature], [TempDiff])])
     et = aggobj._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     df_agg = aggobj.execute(df=df_agg)
 
@@ -224,8 +225,19 @@ def test_anomaly_scores():
 
     print("Executed Anomaly functions on aggregation data")
 
+    df_i = df_i[0:10]   # cut off data
+    spsi = SpectralAnomalyScore(Temperature, 12, spectral)
+    et = spsi._build_entity_type(columns=[Column(Temperature, Float())])
+    spsi._entity_type = et
+    df_i = spsi.execute(df=df_i)
+
+    assert (not df_i.empty)
+
+    print("Executed Anomaly functions on minimal data ", df_i.empty)
+
     pass
 
 
-# uncomment to run from the command line
-#test_anomaly_scores()
+if __name__ == '__main__':
+    test_anomaly_scores()
+
