@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -8,6 +8,8 @@
 #
 # *****************************************************************************
 
+import logging
+import unittest
 from collections import OrderedDict
 import datetime as dt
 import numpy as np
@@ -15,7 +17,7 @@ import pandas as pd
 from sqlalchemy import Column, Float, DateTime
 from iotfunctions.bif import DateDifference
 from iotfunctions.generator import AnomalyGeneratorExtremeValue
-from nose.tools import assert_true
+#from nose.tools import assert_true
 
 # constants
 Temperature = 'Temperature'
@@ -73,11 +75,11 @@ def test_anomaly_generators():
     #print(origins1)
     #print(origins2)
 
-    # assert_true(comp[0])
-    # assert_true(comp[1])
-    # assert_true(comp[2])
+    # assert (comp[0])
+    # assert (comp[1])
+    # assert (comp[2])
 
     pass
 
-# uncomment to run from CLI
-# test_anomaly_generators()
+if __name__ == '__main__':
+    test_anomaly_generators()
