@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -9,6 +9,7 @@
 # *****************************************************************************
 
 import logging
+import unittest
 import numpy as np
 import pandas as pd
 from sklearn.metrics import r2_score
@@ -19,12 +20,13 @@ from iotfunctions.anomaly import GBMRegressor
 from iotfunctions.db import Database
 from iotfunctions.dbtables import FileModelStore
 from iotfunctions.enginelog import EngineLogging
-from nose.tools import assert_true, nottest
+#from nose.tools import assert_true, nottest
 
 # constants
 Temperature = 'TEMP_AIR'
 Humidity = 'HUMIDITY'
 KW = 'KW'
+MyShop = 'MyShop'
 kmeans = 'TemperatureKmeansScore'
 fft = 'TemperatureFFTScore'
 spectral = 'TemperatureSpectralScore'
@@ -33,7 +35,7 @@ gen = 'TemperatureGeneralizedScore'
 
 logger = logging.getLogger('Test Regressor')
 
-@nottest
+#@nottest
 class DatabaseDummy:
     tenant_id = '###_IBM_###'
     db_type = 'db2'
@@ -42,7 +44,7 @@ class DatabaseDummy:
         return
 
 
-@nottest
+#@nottest
 class TestRegressor(BaseEstimatorFunction):
 
     eval_metric = staticmethod(r2_score)
@@ -125,12 +127,16 @@ def test_base_estimator_function():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW)  # no suffix, works on entity type level
+
     df_i = brgi.execute(df=df_i)
     print('Base regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_TESTREGRESSOR.TestRegressor.KW'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('Base regressor - testing training pipeline done ')
 
@@ -147,12 +153,16 @@ def test_base_estimator_function():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW)
+
     df_i = brgi.execute(df=df_i)
     print('Base regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_TESTREGRESSOR.TestRegressor.KW'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.35)
+    assert (mtrc > 0.35)
 
     print('Base regressor - testing training pipeline done ')
 
@@ -169,12 +179,16 @@ def test_base_estimator_function():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW)
+
     df_i = brgi.execute(df=df_i)
     print('Base regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_TESTREGRESSOR.TestRegressor.KW'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.35)
+    assert (mtrc > 0.35)
 
     print('Base regressor - inference done')
 
@@ -190,18 +204,22 @@ def test_base_estimator_function():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW)
+
     df_i = brgi.execute(df=df_i)
     print('Base regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_TESTREGRESSOR.TestRegressor.KW'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.35)
+    assert (mtrc > 0.35)
 
     print('Base regressor - enforce retraining done')
 
     pass
 
 
-# uncomment to run from the command line
-# test_base_estimator_function()
+if __name__ == '__main__':
+    test_base_estimator_function()
 
