@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -8,13 +8,15 @@
 #
 # *****************************************************************************
 
+import logging
+import unittest
 from collections import OrderedDict
 import datetime as dt
 import numpy as np
 import pandas as pd
 from sqlalchemy import Column, Float, DateTime
 from iotfunctions.bif import Coalesce, DateDifference
-from nose.tools import assert_true
+#from nose.tools import assert_true
 
 # constants
 Temperature = 'Temperature'
@@ -97,10 +99,11 @@ def test_base_functions():
     print(origins2)
     print(comp)
 
-    assert_true(comp[0])
-    assert_true(comp[1])
-    assert_true(comp[2])
+    assert (comp[0])
+    assert (comp[1])
+    assert (comp[2])
 
     pass
 
-# test_base_scores()
+if __name__ == '__main__':
+    test_base_functions()
