@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -9,6 +9,7 @@
 # *****************************************************************************
 
 import logging
+import unittest
 import numpy as np
 import pandas as pd
 from sklearn.metrics import r2_score
@@ -18,16 +19,17 @@ from iotfunctions.anomaly import BayesRidgeRegressor, BayesRidgeRegressorExt
 from iotfunctions.db import Database
 from iotfunctions.dbtables import FileModelStore
 from iotfunctions.enginelog import EngineLogging
-from nose.tools import assert_true, nottest
+#from nose.tools import assert_true, nottest
 
 # constants
 Temperature = 'TEMP_AIR'
 Humidity = 'HUMIDITY'
 KW = 'KW'
+MyShop = 'MyShop'
 
 logger = logging.getLogger('Test Regressor')
 
-@nottest
+#@nottest
 class DatabaseDummy:
     tenant_id = '###_IBM_###'
     db_type = 'db2'
@@ -77,6 +79,7 @@ def test_bayes_ridge():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
 
     df_i = brgi.execute(df=df_i)
 
@@ -92,13 +95,16 @@ def test_bayes_ridge():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
 
     df_i = brgi.execute(df=df_i)
     print('Bayes regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_BAYESRIDGEREGRESSOR.BayesRidgeRegressor.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('Bayes regressor - testing training pipeline done ')
 
@@ -117,12 +123,16 @@ def test_bayes_ridge():
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
 
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgi.execute(df=df_i)
     print('Bayes regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_BAYESRIDGEREGRESSOR.BayesRidgeRegressor.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('Bayes regressor - inference done')
 
@@ -139,12 +149,16 @@ def test_bayes_ridge():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgi.execute(df=df_i)
     print('Bayes regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_BAYESRIDGEREGRESSOR.BayesRidgeRegressor.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('Bayes regressor - enforce retraining done')
 
@@ -161,18 +175,21 @@ def test_bayes_ridge():
 
     et = brgei._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgei._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgei.execute(df=df_i)
     print('Bayes regressor ext done')
 
-    mtrc = brgei.active_models['model.TEST_ENTITY_FOR_BAYESRIDGEREGRESSOREXT.BayesRidgeRegressorExt.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('Bayes regressor ext - training done')
 
     pass
 
 
-# uncomment to run from the command line
-# test_bayes_ridge()
-
+if __name__ == '__main__':
+    test_bayes_ridge()
