@@ -1,5 +1,5 @@
 # *****************************************************************************
-# © Copyright IBM Corp. 2018.  All Rights Reserved.
+# © Copyright IBM Corp. 2018, 2022  All Rights Reserved.
 #
 # This program and the accompanying materials
 # are made available under the terms of the Apache V2.0 license
@@ -9,6 +9,7 @@
 # *****************************************************************************
 
 import logging
+import unittest
 import numpy as np
 import pandas as pd
 from sklearn.metrics import r2_score
@@ -18,16 +19,17 @@ from iotfunctions.anomaly import (GBMRegressor, GBMForecaster)
 from iotfunctions.db import Database
 from iotfunctions.dbtables import FileModelStore
 from iotfunctions.enginelog import EngineLogging
-from nose.tools import assert_true, nottest
+#from nose.tools import assert_true, nottest
 
 # constants
 Temperature = 'TEMP_AIR'
 Humidity = 'HUMIDITY'
 KW = 'KW'
+MyShop = 'MyShop'  # entity name
 
 logger = logging.getLogger('Test Regressor')
 
-@nottest
+#@nottest
 class DatabaseDummy:
     tenant_id = '###_IBM_###'
     db_type = 'db2'
@@ -47,7 +49,7 @@ def test_light_gbm():
     df_i = pd.read_csv('./data/RegressionTestData.csv', index_col=False, parse_dates=['DATETIME'])
     df_i = df_i.rename(columns={'DATETIME': 'timestamp'})
 
-    df_i['entity'] = 'MyShop'
+    df_i['entity'] = MyShop
     df_i[Temperature] = pd.to_numeric(df_i[Temperature], errors='coerce')
     df_i[Humidity] = pd.to_numeric(df_i[Humidity], errors='coerce')
 
@@ -81,6 +83,10 @@ def test_light_gbm():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+    print (model_name)
 
     df_i = brgi.execute(df=df_i)
 
@@ -97,13 +103,18 @@ def test_light_gbm():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
 
     df_i = brgi.execute(df=df_i)
     print('lightGBM regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_GBMREGRESSOR.GBMRegressor.KW.MyShop'][0].eval_metric_test
+    #mtrc = brgi.active_models['model.TEST_ENTITY_FOR_GBMREGRESSOR.GBMRegressor.KW.MyShop'][0].eval_metric_test
+
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('lightGBM regressor - testing training pipeline done ')
 
@@ -122,12 +133,15 @@ def test_light_gbm():
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
 
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgi.execute(df=df_i)
     print('lightGBM regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_GBMREGRESSOR.GBMRegressor.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('lightGBM regressor - inference done')
 
@@ -144,12 +158,15 @@ def test_light_gbm():
 
     et = brgi._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgi._entity_type = et
+    brgi._entity_type.name = MyShop
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgi.execute(df=df_i)
     print('lightGBM regressor done')
 
-    mtrc = brgi.active_models['model.TEST_ENTITY_FOR_GBMREGRESSOR.GBMRegressor.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('lightGBM regressor - enforce retraining done')
 
@@ -164,12 +181,15 @@ def test_light_gbm():
 
     et = brgei._build_entity_type(columns=[Column(Temperature, Float())], **jobsettings)
     brgei._entity_type = et
+    brgi._entity_type.name = MyShop
+    model_name = brgi.generate_model_name([Temperature, Humidity], KW, suffix=MyShop)
+
     df_i = brgei.execute(df=df_i)
     print('lightGBM forecaster done')
 
-    mtrc = brgei.active_models['model.TEST_ENTITY_FOR_GBMFORECASTER.GBMForecaster.KW.MyShop'][0].eval_metric_test
+    mtrc = brgi.active_models[model_name][0].eval_metric_test
     print ('Trained model r2 ', mtrc)
-    assert_true(mtrc > 0.4)
+    assert (mtrc > 0.4)
 
     print('lightGBM forecaster - training done')
 
@@ -177,6 +197,5 @@ def test_light_gbm():
     pass
 
 
-# uncomment to run from the command line
-# test_light_gbm()
-
+if __name__ == '__main__':
+    test_light_gbm()
